#!/usr/bin/env python

"""
This is a class which processes the config file and holds all of
the various config options
"""

__cvsversion__ = '$Id: Config.py,v 1.15 2007/12/06 23:32:20 jay Exp $'
__author__ = 'Jay Deiman; jay@splitstreams.com'
__copyright__ = 'Copyright 2007 Jason Deiman'
__license__ = 'GPLv3'
__all__ = ['ConfTypeError' , 'Config']

import syslog , re , pwd , grp , os
from MySQLDB import MySQLDB
from SQLiteDB import SQLiteDB
from NullRtRules import NullRtRules
from IPTablesRules import IPTablesRules
from PfRules import PfRules

class ConfTypeError(Exception):
    pass

class Config(object):
    """
    Class to hold all of the configuration options.  Contains
    defaults as well as the ability to update in memory settings
    """
    
    lineRegex = re.compile(r'^\s*([^#=]+)=\s*([^#]+)')
    
    def __init__(self , daemonize=False , confFile=None , whiteFile=None , 
                 regexFile=None):
        """
        Set all the default variables and read the config file
        """
        self._confFile = (confFile , 
                        '/etc/dictdefence/dictdefence.conf' )[confFile == None]
        # Valid db types
        self._dbTypes = {'sqlite': SQLiteDB , 
                         'mysql': MySQLDB}
        # Default db type
        self._dbType = 'sqlite'
        # Valid reader types
        self._readFiles = ['/var/log/messages']
        # Valid rules
        self._ruleTypes = {'iptables': IPTablesRules , 
                           'pf': PfRules , 
                           'nullroute': NullRtRules}
        self._ruleType = 'nullroute'
        self._ruleCmdPath = '/sbin/route'
        self._pfAnchorName = 'scriptkiddies'
        # DB specific configs (for MySql)
        self._dbName = None
        self._dbPort = 3306
        self._dbUnixSocket = None
        self._dbHost = 'localhost'
        self._dbUser = None
        self._dbPass = None
        # For SQLite only
        self._dbFile = '/var/db/dictdefence/db.sqlite'
        # The thresholds for blocking, starting with the number of failures
        # needed to trigger a block
        self._numFails = 4
        # Period of time in which the numFails must occur
        self._timePeriod = 600
        # How long to block for, default is one day
        self._blockTime = 86400
        self._defaultWhiteFile = '/etc/dictdefence/dictdefence.whitelist'
        self._defaultRegexFile = '/etc/dictdefence/dictdefence.regex'
        # Whitelist file
        self._whiteFile = (whiteFile , 
                self._defaultWhiteFile)[whiteFile == None]
        # Regex file
        self._regexFile = (regexFile , 
                self._defaultRegexFile)[regexFile == None]
        # Logging facilities
        self._logFacility = syslog.LOG_AUTH
        self._logPriority = syslog.LOG_WARNING
        self._logOptions = syslog.LOG_PID
        self._sockPath = '/tmp/dictdef.sock'
        self._daemonize = daemonize
        self.readConfFile()
        
    def readConfFile(self):
        """
        Method to read the conf file and parse for entries
        """
        cf = open(self._confFile)
        while True:
            line = cf.readline().strip()
            if not line: break
            res = self._procLine(line)
            if not res: continue
            # Set the appropriate var based on the line entry
            if res[0] in ('LogPriority' , 'LogFacility'):
                exec('self.set%s(syslog.%s)' % res)
            elif res[0] == 'LogOptions':
                opts = []
                for opt in res[1].split('|'):
                    opts.append('syslog.' + opt.strip())
                opts = '|'.join(opts)
                exec('self.set%s(%s)' , (res[0] , opts))
            else: 
                exec('self.set%s("%s")' % res)
        cf.close()
        
    def _procLine(self , line):
        """
        This private method extracts the key/value pair from a given
        line and returns them as a tuple of 2 values or None if it is
        a comment or invalid line
        """
        m = self.lineRegex.match(line)
        if not m: return None
        key = m.group(1).strip().strip("'").strip('"')
        val = m.group(2).strip().strip("'").strip('"')
        return (key , val)
    
    # Getters and setters
    def getConfFile(self):
        """ Path to the main config file """
        return self._confFile
    def setConfFile(self , confFile):
        self._confFile = confFile
    ConfFile = property(getConfFile , setConfFile)
    
    def getDBTypes(self):
        """ The available db types """
        return self._dbTypes
    DBTypes = property(getDBTypes)
    
    def getDBType(self):
        """ The current db type """
        return self._dbType
    def setDBType(self , type):
        type = type.lower()
        if type not in self._dbTypes.keys():
            raise ConfTypeError , 'Invalid DB type: %s' % type
        self._dbType = type
    DBType = property(getDBType , setDBType)
    
    def getCBDBType(self):
        """ Returns the callback to create the correct DB type """
        return self._dbTypes[self._dbType]
    CBDBType = property(getCBDBType)
    
    def getReadFiles(self):
        """ The file or fifo to read from """
        return self._readFiles
    def setReadFiles(self , filepaths):
        self._readFiles = [f.strip() for f in filepaths.split(';')]
    ReadFiles = property(getReadFiles , setReadFiles)
    
    def getRuleTypes(self):
        """ The list of available rule sets """
        return self._ruleTypes
    RuleTypes = property(getRuleTypes)
    
    def getRuleType(self):
        """ The currently specified rule type """
        return self._ruleType
    def setRuleType(self , type):
        type = type.lower()
        if type not in self._ruleTypes.keys():
            raise ConfTypeError , 'Invalid rule set type: %s' % type
        self._ruleType = type
    RuleType = property(getRuleType , setRuleType)
    
    def getCBRuleType(self):
        """ Returns the callback to create the correct Rule type """
        return self._ruleTypes[self._ruleType]
    CBRuleType = property(getCBRuleType)
    
    def getRuleCmdPath(self):
        return self._ruleCmdPath
    def setRuleCmdPath(self , path):
        self._ruleCmdPath = path
    RuleCmdPath = property(getRuleCmdPath , setRuleCmdPath)
    
    def getPfAnchorName(self):
        return self._pfAnchorName
    def setPfAnchorName(self , name):
        self._pfAnchorName = name
    PfAnchorName = property(getPfAnchorName , setPfAnchorName)
    
    def getSudoPath(self):
        return self._sudoPath
    def setSudoPath(self , path):
        self._sudoPath = path
    SudoPath = property(getSudoPath , setSudoPath)
    
    def getExtIface(self):
        return self._extIface
    def setExtIface(self , name):
        self._extIface = name
    ExtIface = property(getExtIface , setExtIface)
    
    def getDBName(self):
        """ The name of the database; not used with SQLite """
        return self._dbName
    def setDBName(self , name):
        self._dbName = name
    DBName = property(getDBName , setDBName)
    
    def getDBPort(self):
        return self._dbPort
    def setDBPort(self , port):
        self._dbPort = int(port)
    DBPort = property(getDBPort , setDBPort)
    
    def getDBUnixSocket(self):
        return self._dbUnixSocket
    def setDBUnixSocket(self , sock):
        self._dbUnixSocket = sock
    DBUnixSocket = property(getDBUnixSocket , setDBUnixSocket)
    
    def getDBHost(self):
        """ 
        The hostname or IP of the database server; not used 
        with SQLite
        """
        return self._dbHost
    def setDBHost(self , host):
        self._dbHost = host.lower()
    DBHost = property(getDBHost , setDBHost)
    
    def getDBUser(self):
        """ The username for the database; not used with SQLite """
        return self._dbUser
    def setDBUser(self , user):
        self._dbUser = user
    DBUser = property(getDBUser , setDBUser)
    
    def getDBPass(self):
        """ 
        Returns the password for the database; not used with SQLite
        """
        return self._dbPass
    def setDBPass(self , password):
        self._dbPass = password
    DBPass = property(getDBPass , setDBPass)
    
    def getDBFile(self):
        """ The path to the database file; only valid with SQLite """
        return self._dbFile
    def setDBFile(self , filename):
        self._dbFile = filename
    DBFile = property(getDBFile , setDBFile)
    
    def getNumFails(self):
        """ The number of login failures before the IP is blocked """
        return self._numFails
    def setNumFails(self , num):
        self._numFails = int(num)
    NumFails = property(getNumFails , setNumFails)
    
    def getTimePeriod(self):
        return self._timePeriod
    def setTimePeriod(self , num):
        self._timePeriod = int(num)
    TimePeriod = property(getTimePeriod , setTimePeriod)
    
    def getBlockTime(self):
        return self._blockTime
    def setBlockTime(self , num):
        self._blockTime = int(num)
    BlockTime = property(getBlockTime , setBlockTime)
    
    def getWhiteFile(self):
        """ The path to the whitelist file """
        return self._whiteFile
    def setWhiteFile(self , path):
        if self._whiteFile == self._defaultWhiteFile:
            self._whiteFile = path
    WhiteFile = property(getWhiteFile , setWhiteFile)
    
    def getRegexFile(self):
        return self._regexFile
    def setRegexFile(self , path):
        if self._regexFile == self._defaultRegexFile:
            self._regexFile = path
    RegexFile = property(getRegexFile , setRegexFile)
    
    def getLogFacility(self):
        """ 
        The log facility that this uses; see 'man syslog' for 
        more info
        """
        return self._logFacility
    def setLogFacility(self , fac):
        if fac not in (syslog.LOG_KERN , syslog.LOG_USER , syslog.LOG_MAIL ,
                       syslog.LOG_DAEMON , syslog.LOG_AUTH , syslog.LOG_LPR ,
                       syslog.LOG_NEWS , syslog.LOG_UUCP , syslog.LOG_CRON ,
                       syslog.LOG_LOCAL0 , syslog.LOG_LOCAL1 , 
                       syslog.LOG_LOCAL2 , syslog.LOG_LOCAL3 ,
                       syslog.LOG_LOCAL4 , syslog.LOG_LOCAL5 ,
                       syslog.LOG_LOCAL6 , syslog.LOG_LOCAL7):
            raise ConfTypeError , 'Invalid log facility: %s' % fac
        self._logFacility = fac
    LogFacility = property(getLogFacility , setLogFacility)
    
    def getLogPriority(self):
        """ 
        The log priority that this uses; see 'man syslog' for
        more info
        """
        return self._logPriority
    def setLogPriority(self , priority):
        if priority not in (syslog.LOG_EMERG , syslog.LOG_ALERT , 
                            syslog.LOG_CRIT , syslog.LOG_ERR ,
                            syslog.LOG_WARNING , syslog.LOG_NOTICE ,
                            syslog.LOG_INFO , syslog.LOG_DEBUG):
            raise ConfTypeError , 'Invalid log priority: %s' % priority
        self._logPriority = priority
    LogPriority = property(getLogPriority , setLogPriority)
    
    def getLogOptions(self):
        """ The syslogging options; see 'man syslog' for more info """
        return self._logOptions
    def setLogOptions(self , opts):
        self._logOptions = int(opts)
    LogOptions = property(getLogOptions , setLogOptions)
    
    def getSockPath(self):
        return self._sockPath
    def setSockPath(self , path):
        if os.path.exists(os.path.dirname(path)):
            self._sockPath = path
    SockPath = property(getSockPath , setSockPath)
    
    def getDaemonize(self):
        return self._daemonize
    def setDaemonize(self , bool):
        self._daemonize = bool
    Daemonize = property(getDaemonize , setDaemonize)